"""
Your project description goes here.
"""

from __future__ import absolute_import, unicode_literals

__version__ = "1.8.1"

default_app_config = "enterprise.apps.EnterpriseConfig"  # pylint: disable=invalid-name



# -*- coding: utf-8 -*-
"""
URL definitions for enterprise API endpoint.
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import include, url

urlpatterns = [
    url(r'^v1/', include('enterprise.api.v1.urls'), name='api')
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0027_remove_account_level_consent'),
    ]

    operations = [
        migrations.AddField(
            model_name='enrollmentnotificationemailtemplate',
            name='enterprise_customer',
            field=models.OneToOneField(related_name='enterprise_enrollment_template', default=None, to='enterprise.EnterpriseCustomer'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='enterprise_customer',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, db_constraint=False, blank=True, to='enterprise.EnterpriseCustomer', null=True),
        ),
        migrations.AlterField(
            model_name='enrollmentnotificationemailtemplate',
            name='site',
            field=models.OneToOneField(related_name='enterprise_enrollment_template', null=True, blank=True, to='sites.Site'),
        ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0026_make_require_account_level_consent_nullable'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicaluserdatasharingconsentaudit',
            name='history_user',
        ),
        migrations.RemoveField(
            model_name='historicaluserdatasharingconsentaudit',
            name='user',
        ),
        migrations.RemoveField(
            model_name='userdatasharingconsentaudit',
            name='user',
        ),
        migrations.RemoveField(
            model_name='enterprisecourseenrollment',
            name='consent_granted',
        ),
        migrations.RemoveField(
            model_name='enterprisecustomer',
            name='require_account_level_consent',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecourseenrollment',
            name='consent_granted',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomer',
            name='require_account_level_consent',
        ),
        migrations.DeleteModel(
            name='HistoricalUserDataSharingConsentAudit',
        ),
        migrations.DeleteModel(
            name='UserDataSharingConsentAudit',
        ),
    ]



# -*- coding: utf-8 -*-

""" Custom API permissions. """

from __future__ import absolute_import, unicode_literals

from rest_framework import permissions


class IsInEnterpriseGroup(permissions.BasePermission):
    """
    Find out if the requesting user belongs to a django group meant for granting access to an enterprise feature.
    This check applies to both staff and non-staff users.
    """
    ALLOWED_API_GROUPS = []  # pylint: disable=invalid-name
    message = u'User is not allowed to access the view.'

    def has_permission(self, request, view):
        return (
            super(IsInEnterpriseGroup, self).has_permission(request, view) and (
                request.user.groups.filter(name__in=self.ALLOWED_API_GROUPS).exists() or
                request.user.groups.filter(name__in=request.query_params.getlist('permissions')).exists()
            )
        )



"""
Rules needed to restrict access to the enterprise data api.
"""
from __future__ import absolute_import, unicode_literals

import crum
import rules
from edx_rbac.utils import request_user_has_implicit_access_via_jwt, user_has_access_via_database
from edx_rest_framework_extensions.auth.jwt.authentication import get_decoded_jwt_from_auth
from edx_rest_framework_extensions.auth.jwt.cookies import get_decoded_jwt

from enterprise.constants import (
    ENTERPRISE_CATALOG_ADMIN_ROLE,
    ENTERPRISE_DASHBOARD_ADMIN_ROLE,
    ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
)
from enterprise.models import EnterpriseFeatureUserRoleAssignment


@rules.predicate
def has_implicit_access_to_dashboard(user, obj):  # pylint: disable=unused-argument
    """
    Check that if request user has implicit access to `ENTERPRISE_DASHBOARD_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    request = crum.get_current_request()
    decoded_jwt = get_decoded_jwt(request) or get_decoded_jwt_from_auth(request)
    return request_user_has_implicit_access_via_jwt(decoded_jwt, ENTERPRISE_DASHBOARD_ADMIN_ROLE)


@rules.predicate
def has_explicit_access_to_dashboard(user, obj):  # pylint: disable=unused-argument
    """
    Check that if request user has explicit access to `ENTERPRISE_DASHBOARD_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    return user_has_access_via_database(
        user,
        ENTERPRISE_DASHBOARD_ADMIN_ROLE,
        EnterpriseFeatureUserRoleAssignment
    )


@rules.predicate
def has_implicit_access_to_catalog(user, obj):  # pylint: disable=unused-argument
    """
    Check that if request user has implicit access to `ENTERPRISE_CATALOG_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    request = crum.get_current_request()
    decoded_jwt = get_decoded_jwt(request) or get_decoded_jwt_from_auth(request)
    return request_user_has_implicit_access_via_jwt(decoded_jwt, ENTERPRISE_CATALOG_ADMIN_ROLE, obj)


@rules.predicate
def has_explicit_access_to_catalog(user, obj):
    """
    Check that if request user has explicit access to `ENTERPRISE_CATALOG_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    return user_has_access_via_database(
        user,
        ENTERPRISE_CATALOG_ADMIN_ROLE,
        EnterpriseFeatureUserRoleAssignment,
        obj
    )


@rules.predicate
def has_implicit_access_to_enrollment_api(user, obj):  # pylint: disable=unused-argument
    """
    Check that if request user has implicit access to `ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    request = crum.get_current_request()
    decoded_jwt = get_decoded_jwt(request) or get_decoded_jwt_from_auth(request)
    return request_user_has_implicit_access_via_jwt(decoded_jwt, ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE, obj)


@rules.predicate
def has_explicit_access_to_enrollment_api(user, obj):
    """
    Check that if request user has explicit access to `ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE` feature role.

    Returns:
        boolean: whether the request user has access or not
    """
    return user_has_access_via_database(
        user,
        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
        EnterpriseFeatureUserRoleAssignment,
        obj
    )


rules.add_perm('enterprise.can_access_admin_dashboard',
               has_implicit_access_to_dashboard | has_explicit_access_to_dashboard)

rules.add_perm('enterprise.can_view_catalog',
               has_implicit_access_to_catalog | has_explicit_access_to_catalog)

rules.add_perm('enterprise.can_enroll_learners',
               has_implicit_access_to_enrollment_api | has_explicit_access_to_enrollment_api)



# -*- coding: utf-8 -*-

"""
Models for xAPI.
"""
from __future__ import absolute_import, unicode_literals

import base64

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from model_utils.models import TimeStampedModel

from enterprise.models import EnterpriseCustomer


@python_2_unicode_compatible
class XAPILRSConfiguration(TimeStampedModel):
    """
    xAPI LRS configurations.

    .. no_pii:
    """

    enterprise_customer = models.OneToOneField(
        EnterpriseCustomer,
        blank=False,
        null=False,
        help_text=_('Enterprise Customer associated with the configuration.'),
    )
    version = models.CharField(max_length=16, default='1.0.1', help_text=_('Version of xAPI.'))
    endpoint = models.URLField(help_text=_('URL of the LRS.'))
    key = models.CharField(max_length=255, verbose_name="Client ID", help_text=_('Key of xAPI LRS.'))
    secret = models.CharField(max_length=255, verbose_name="Client Secret", help_text=_('secret of xAPI LRS.'))
    active = models.BooleanField(
        blank=False,
        null=False,
        help_text=_('Is this configuration active?'),
    )

    class Meta:
        app_label = 'xapi'

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return '<XAPILRSConfiguration for Enterprise {enterprise_name}>'.format(
            enterprise_name=self.enterprise_customer.name
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @property
    def authorization_header(self):
        """
        Authorization header for authenticating requests to LRS.
        """
        return 'Basic {}'.format(
            base64.b64encode('{key}:{secret}'.format(key=self.key, secret=self.secret).encode()).decode()
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0042_replace_sensitive_sso_username'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enrollmentnotificationemailtemplate',
            options={'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecourseenrollment',
            options={'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomer',
            options={'verbose_name': 'Enterprise Customer', 'verbose_name_plural': 'Enterprise Customers', 'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomerbrandingconfiguration',
            options={'verbose_name': 'Branding Configuration', 'verbose_name_plural': 'Branding Configurations', 'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomercatalog',
            options={'verbose_name': 'Enterprise Customer Catalog', 'verbose_name_plural': 'Enterprise Customer Catalogs', 'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomerentitlement',
            options={'verbose_name': 'Enterprise Customer Entitlement', 'verbose_name_plural': 'Enterprise Customer Entitlements', 'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomeridentityprovider',
            options={'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomerreportingconfiguration',
            options={'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='enterprisecustomeruser',
            options={'verbose_name': 'Enterprise Customer Learner', 'verbose_name_plural': 'Enterprise Customer Learners', 'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='pendingenrollment',
            options={'ordering': ['created']},
        ),
        migrations.AlterModelOptions(
            name='pendingenterprisecustomeruser',
            options={'ordering': ['created']},
        ),
    ]
